@@ -25,6 +25,14 @@ function changeState(myField) {
   document.getElementById(myField).disabled=(document.getElementById(myField).disabled)?false:true;
 }
 
+function changeSelectState(triggerField, myField) {
+  if (document.getElementById(triggerField).value != 2){
+	  document.getElementById(myField).disabled= true;
+  } else {
+	  document.getElementById(myField).disabled= false;
+  }
+}
+
 document.onkeydown= keyDown; // work together to analyze keystrokes
 if (netscape){
   document.captureEvents(Event.KEYDOWN|Event.KEYUP);
