@@ -30,8 +30,13 @@ if (!isset($_SESSION["ui"])){
 	exit;
 }
 $ui= $_SESSION["ui"];
+$config= $_SESSION["config"];
 gosa_log ("User \"".$ui->username."\" logged out");
-session_destroy ();
+
+# Remove all locks of this user
+del_user_locks($ui->dn);
+
+@session_destroy ();
 
 /* Go back to the base via header */
 header ("Location: index.php");
