@@ -63,7 +63,7 @@ $config= $_SESSION['config'];
 if (isset ($config->data['MAIN']['COMPILE'])){
         $smarty->compile_dir= $config->data['MAIN']['COMPILE'];
 } else {
-        $smarty->compile_dir= '/var/spool/gosa';
+        $smarty->compile_dir= '/var/spool/gosa/';
 }
 
 /* Language setup */
@@ -267,4 +267,5 @@ if (isset($_SESSION['errors'])){
 $smarty->display(get_template_path('framework.tpl'));
 $_SESSION['plist']= $plist;
 
+// vim:tabstop=4:expandtab:shiftwidth=4:filetype=php:syntax:
 ?>
